unit DBGridPrint;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  dbgrids, Printers, PrintHtm, Mask;

type
  TDBGridPrint = class(TComponent)
  private
    FHandle  : THandle;
    FPreview : Boolean;
    FDbGrid  : TDBGrid;
    FTitel   : String;
    FFileName: String;
    FOrientation : TPrinterOrientation;
  protected
    { Protected declarations }
  public
    constructor Create(AOwner: TComponent); override;
    function  Execute : Integer;
    property Handle: THandle read FHandle;
  published
    property Preview : Boolean read FPreview  write FPreview;
    property DbGrid  : TDBGrid read FDbGrid   write FDbGrid;
    property Titel   : String  read FTitel    write FTitel;
    property FileName: String  read FFileName write FFileName;
    property Orientation :TPrinterOrientation read FOrientation write FOrientation default poPortrait;
  end;

implementation

uses
  ShellAPI, DB;

constructor TDBGridPrint.Create(AOwner: TComponent);
begin
  Inherited Create(AOwner);
  FHandle := 0;
end;

function TDBGridPrint.Execute : Integer;
Var
  St        : Array[0..255] of char;
  F         : TextFile;
  lCnt      : Integer;
  lIndex    : Integer;
  lStr      : String;
  SavePlace : TBookmark;

  function MinimalSpace(aStr : String): String;
  begin
   if aStr = '' then
     aStr := '-';
    Result := aStr
  end;

  function StrToMoney(aStr: String): String;
  Var
    lMoney :  Extended;
  begin
     lMoney := StrToFloat(aStr);
     Result := Format('%8.2m', [lMoney]);
  end;

begin
  Result := 0;
  AssignFile(F, FileName);
{$I-}
  Rewrite(F);
{$I+}
  if IOResult <> 0 then
  Begin
     ShowMessage('Failed to create printerfile ' + FileName);
     exit;
  end;
  Writeln(F,'<HTML>');
  Writeln(F,'<HEAD>');
  {Titel}
  Writeln(F,'<B><FONT SIZE=5><P>' + Titel +'</P></B></FONT>');
  Writeln(F,'</HEAD>');
  Writeln(F,'<BODY>');
  {Bepaal Tabelwidth}
  Writeln(F,'<TABLE CELLSPACING=1 BORDER=1 CELLPADDING=1 WIDTH= 100%');
  Writeln(F,'<TR>');

  SavePlace := DbGrid.DataSource.DataSet.GetBookmark;
  DbGrid.DataSource.DataSet.DisableControls;

  Writeln(F,'<FONT><FONT FACE="Arial" SIZE=1>');
  for lCnt := 0 to DbGrid.Columns.Count -1 do
  begin
    if DbGrid.Columns.Items[lCnt].field.Tag = 0 then
    begin
      lStr := DbGrid.Columns.Items[lCnt].Title.Caption;
      Writeln(F,'<TD VALIGN="TOP"><B><P>' + lStr + '</P></B></TD>');
    end;
  end;
  Writeln(F,'</TR>');

  DbGrid.DataSource.DataSet.First;
  while not DbGrid.DataSource.DataSet.Eof do
  begin
    Writeln(F,'<TR>');
    for lIndex := 0 to DbGrid.Columns.Count -1 do
    begin
      if DbGrid.Columns.Items[lIndex].Field.Tag = 0 then
      begin
        WriteLn(F,'<TD VALIGN="TOP">');
        lStr := DbGrid.Columns.Items[lIndex].Field.AsString;
        if DbGrid.Columns.Items[lIndex].Field.DataType in
           [ftCurrency, ftInteger, ftFloat, ftLargeInt, ftWord] then // etc...
        begin
          if (DbGrid.Columns.Items[lIndex].Field.DataType in [ftCurrency]) and
             (lStr > '') then lStr := StrToMoney(lStr);
          Writeln(F,'<P ALIGN="RIGHT">' + MinimalSpace(lStr) + '</TD>');
        end
        else
          Writeln(F,'<P>' + MinimalSpace(lStr) + '</TD>');
      end;
    end;
    Writeln(F,'</TR>');
    DbGrid.DataSource.DataSet.Next;
  end;
  DbGrid.DataSource.DataSet.GotoBookmark(SavePlace);
  DbGrid.DataSource.DataSet.EnableControls;
  Writeln(F,'</FONT>');
  Writeln(F,'</TABLE>');
  Writeln(F,'</BODY>');
  Writeln(F,'</HTML>');
  CloseFile(F);

  Printer.Orientation := Orientation;

  if Preview then
     DoPrintHtm(FileName)
  else
    Result := ShellExecute(Handle,'Print',
                 StrPCopy(St, FileName),nil,nil,SW_SHOW);

{  Case Result of 0	The operating system is out of memory or resources.
                 ERROR_FILE_NOT_FOUND	The specified file was not found.
                 ERROR_PATH_NOT_FOUND	The specified path was not found.
                 ERROR_BAD_FORMAT	The .EXE file is invalid (non-Win32 .EXE or error in .EXE image).
                 SE_ERR_ACCESSDENIED	The operating system denied access to the specified file.
                 SE_ERR_ASSOCINCOMPLETE	The filename association is incomplete or invalid.
                 SE_ERR_DDEBUSY	The DDE transaction could not be completed because other DDE transactions were being processed.
                 SE_ERR_DDEFAIL	The DDE transaction failed.
                 SE_ERR_DDETIMEOUT	The DDE transaction could not be completed because the request timed out.
                 SE_ERR_DLLNOTFOUND	The specified dynamic-link library was not found.
                 SE_ERR_FNF	The specified file was not found.
                 SE_ERR_NOASSOC	There is no application associated with the given filename extension.
                 SE_ERR_OOM	There was not enough memory to complete the operation.
                 SE_ERR_PNF	The specified path was not found.
                 SE_ERR_SHARE	A sharing violation occurred.    }
end;

end.
